//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 16.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

package gwe.sql;

import java.net.*;
import java.io.*;
import java.util.HashMap;
import gwe.sql.gweMysqlColumn;
import gwe.sql.gweMysqlException;


// Class to read/write packets to the mysql server

class gweMysqlPacket {
  DataInputStream sin;
  DataOutputStream sout;
  byte packetSequence;
  int bpos_w, bpos_r, packetLength_r;
  public boolean TRACING=false;
  public final int HEADERLENGTH=4;
  public final int MAXBUFFER=65535;
  byte buffer_w[]=new byte[MAXBUFFER];
  byte buffer_r[]=new byte[MAXBUFFER];
  String encoding;
  gweNativeUnicodeMapper mapperclass;

  // Set up given an input and output datastream
  public gweMysqlPacket(DataInputStream i, DataOutputStream o, String enc, gweNativeUnicodeMapper mapper) {
    sin=i;
    sout=o;
    encoding = enc;
    mapperclass = mapper;
  }

  // Set to -1; packetSequence gets ++'ed before sending
  void resetPacketSequence() {
    packetSequence=-1;
  }

  // Read in next packet; get upset if need be
  void next() throws gweMysqlException {
    try {
      int i;
      byte[] b3 = new byte[3];

      // [3: packetLength] [packetSequence]
      for (i=0; i<3; i++) b3[i]=sin.readByte();
      packetLength_r=(int) (ub(b3[0])+(256*ub(b3[1]))+(256*256*ub(b3[2])));
      packetSequence=sin.readByte();

      if (TRACING) {
        System.out.println("Packet length = "+packetLength_r);
        System.out.println("Packet sequence = "+packetSequence);
      }

      // Get the data (block if necessary - thread higher up the us)
      if( buffer_r.length < packetLength_r+1 ) // 1=terminator(0)
      {
        buffer_r = new byte[packetLength_r+1]; // 1=terminator(0)
      }
      sin.readFully(buffer_r, 0, packetLength_r);
      buffer_r[packetLength_r]=0;
      bpos_r=0;

      if (TRACING) {
        for (int j=0; j<packetLength_r; j++) {
          System.out.print(j+" : "+ub(buffer_r[j])+"  : ");
          if (buffer_r[j]>30) System.out.println((char)buffer_r[j]);
          else System.out.println("");
        }

      }
    } catch (Exception e) {throw new gweMysqlException("gweMysql.Next: "+e);}
  }


  // Return unsigned byte as int
  int ub(byte b) {
    return b<0?(int)(256+b):b;
  }

  // Where are we?
  int currentPos_r() {
    return bpos_r;
  }

  // Set position withtin packet data
  void setPos_r(int p) {
    bpos_r=p;
  }

  // Read byte at current position
  byte getCurDataPos_r() {
    return buffer_r[bpos_r];
  }

  byte readByte() {
    return buffer_r[bpos_r++];
  }

  byte[] readByteBlock(int l) {
    byte[] block = new byte[l];
    if( l > 0 )
    {
      System.arraycopy(buffer_r, bpos_r, block, 0, l);
      bpos_r += l;
    }
    return block;
  }

  int read2Bytes() {
    return (int)(ub(buffer_r[bpos_r++])+(256*ub(buffer_r[bpos_r++])));
  }

  int read3Bytes() {
    return (int)(ub(buffer_r[bpos_r++])+(256*ub(buffer_r[bpos_r++]))+(256*256*ub(buffer_r[bpos_r++])));
  }

  long read4Bytes() {
    return (long)(ub(buffer_r[bpos_r++])+(256*ub(buffer_r[bpos_r++]))+(256*256*ub(buffer_r[bpos_r++]))+(256*256*256*ub(buffer_r[bpos_r++])));
  }


  // Read n bytes depending
  int readnBytes() {
    switch(ub(buffer_r[bpos_r++])) {
      case 1 : return ub(buffer_r[bpos_r++]);
      case 2 : return this.read2Bytes();
      case 3 : return this.read3Bytes();
      case 4 : return (int)this.read4Bytes();
      default : return 255;
    }
  }

  // read [:length]
  long readLength() {
    //System.out.println("flag:" + ub(buffer_r[bpos_r]));
    switch(ub(buffer_r[bpos_r])) {
      case 251 : {bpos_r++; return (long) -1; } // for NULL FIELD!!
      case 252 : {bpos_r++; return (long) read2Bytes();}
      case 253 : {bpos_r++; return (long) read3Bytes();}
      case 254 : {bpos_r++; return (long) read4Bytes();}
      default  : return (long) ub(buffer_r[bpos_r++]);
    }
  }


  // Read null-terminated string
  String readString() {
    gweMysqlStringBuffer b = new gweMysqlStringBuffer();
    int i=bpos_r;
    while(buffer_r[i]!=0) b.append((char)buffer_r[i++]);
    bpos_r=i+1;
    return b.toString();
  }

  // Read given-length string
  String readLenString() {
    long len = this.readLength();
    if (len==-1) return null; // for NULL FIELD!!
    if (len==0) return "";
    gweMysqlStringBuffer b = new gweMysqlStringBuffer((int)len);
    for (int i=0; i<len; i++) {
      if (buffer_r[bpos_r]==0) break;
      else b.append((char)buffer_r[bpos_r++]);
    }
    return b.toString();
  }

  // Is this the last packet?
  boolean isLastDataPacket_r() {
    return ((packetLength_r==1)&&(ub(buffer_r[0])==254));
  }

  // Reset position to start of packet data
  void clear_w() {
    bpos_w=HEADERLENGTH;
  }

  // Send packet
  void send() {
    try {
      int l=bpos_w;
      bpos_w=0;
      packetSequence++;
      this.write3Bytes(l-HEADERLENGTH);
      this.writeByte(packetSequence);

      if (TRACING) {
        for (int j=0; j<l; j++) {
          System.out.print(j+" : "+ub(buffer_w[j])+"  : ");
          if (buffer_w[j]>30) System.out.println((char)buffer_w[j]);
          else System.out.println("");
        }
      }

      sout.write(buffer_w, 0, l);
      sout.flush();
    } catch (Exception e) {System.out.println("send() Error : "+e);}
  }

  void writeByte(byte b) {
    buffer_w[bpos_w++]=b;
  }

  void write2Bytes(int i) {
    int r;
    r=i%256;
    buffer_w[bpos_w++]=(byte)r;
    i=i/256;
    buffer_w[bpos_w++]=(byte)i;
  }

  void write3Bytes(int i) {
    int r;
    r=i%256;
    buffer_w[bpos_w++]=(byte)r;
    i=i/256;
    r=i%256;
    buffer_w[bpos_w++]=(byte)r;
    i=i/256;
    buffer_w[bpos_w++]=(byte)i;
  }

  void write4Bytes(long i) {
    long r;
    r=i%256;
    buffer_w[bpos_w++]=(byte)r;
    i=i/256;
    r=i%256;
    buffer_w[bpos_w++]=(byte)r;
    i=i/256;
    r=i%256;
    buffer_w[bpos_w++]=(byte)r;
    i=i/256;
    buffer_w[bpos_w++]=(byte)i;
  }

  // Write null-terminated string
  void writeString(String s) {
    int l = s.length();
    for(int i=0; i<l; i++) buffer_w[bpos_w++]=(byte)s.charAt(i);
    buffer_w[bpos_w++]=0;
  }

  // Write string, with no termination
  void writeStringNoNull(String s) {
//    int l = s.length();
//    for(int i=0; i<l; i++) buffer[bpos++]=(byte)s.charAt(i);
    try{
      byte[] temp_bytes;
      if( mapperclass != null )
      {
        char[] cbuf = new char[s.length()];
        s.getChars(0, s.length(), cbuf, 0);
        temp_bytes = mapperclass.Unicode2Native(cbuf);
      }
      else
      {
        temp_bytes = s.getBytes(encoding);
      }
      int l = temp_bytes.length;
      System.arraycopy(temp_bytes, 0, buffer_w, bpos_w, l);
      bpos_w += l;
    }
    catch(UnsupportedEncodingException e){
      int l = s.length();
      for(int i=0; i<l; i++) buffer_w[bpos_w++]=(byte)s.charAt(i);
    }
  }

  // ---------------------------------------------------------------------

  // Send packet
  void sendHeader(int sql_length) throws gweMysqlException {
    bpos_w = 0;
    packetSequence = 0;
    this.write3Bytes(1 + sql_length); // 1=(byte)gweMysql.QUERY
    this.writeByte(packetSequence);
    this.writeByte((byte)gweMysql.QUERY);
    try {
      sout.write(buffer_w, 0, HEADERLENGTH + 1); // 1=(byte)gweMysql.QUERY
      sout.flush();
      bpos_w = 0;
    }
    catch (IOException io_e) {
      throw new gweMysqlException("gweMysqlPacket.sendHeader: "+io_e);
    }
  }

  // Write string, with no termination and Send Buffer
  void writeStringNoNullandSendBuffer(String s) throws gweMysqlException {
    try{
      byte[] temp_bytes;
      if( mapperclass != null )
      {
        char[] cbuf = new char[s.length()];
        s.getChars(0, s.length(), cbuf, 0);
        temp_bytes = mapperclass.Unicode2Native(cbuf);
      }
      else
      {
        temp_bytes = s.getBytes(encoding);
      }
      int l = temp_bytes.length;
      int w = buffer_w.length;
      for(int i=0; i<l; i++)
      {
        buffer_w[bpos_w++] = temp_bytes[i];
        if( bpos_w == w )
        {
          flushSendBuffer();
        }
      }
    }
    catch(UnsupportedEncodingException e){
      int l = s.length();
      int w = buffer_w.length;
      for(int i=0; i<l; i++)
      {
        buffer_w[bpos_w++] = (byte)s.charAt(i);
        if( bpos_w == w )
        {
          flushSendBuffer();
        }
      }
    }
  }

  // Write string, with no termination and Send Buffer
  void writeStringNoNullandSendBufferEx(String s) throws gweMysqlException {
    int l = s.length();
    int w = buffer_w.length;
    for(int i=0; i<l; i++)
    {
      buffer_w[bpos_w++] = (byte)s.charAt(i);
      if( bpos_w == w )
      {
        flushSendBuffer();
      }
    }
  }

  // Flush Send Buffer
  void flushSendBuffer() throws gweMysqlException {
    try {
      sout.write(buffer_w, 0, bpos_w);
      sout.flush();
      bpos_w = 0;
    }
    catch(IOException io_e) {
      throw new gweMysqlException("gweMysqlPacket.writeStringNoNullandSendBuffer: "+io_e);
    }
  }
}


public class gweMysql {

  // Constants defined from mysql
  public static final int SLEEP=0;
  public static final int QUIT=1;
  public static final int INIT_DB=2;
  public static final int QUERY=3;
  public static final int FIELD_LIST=4;
  public static final int CREATE_DB=5;
  public static final int DROP_DB=6;
  public static final int RELOAD=7;
  public static final int SHUTDOWN=8;
  public static final int STATISTICS=9;
  public static final int PROCESS_INFO=10;
  public static final int CONNECT=11;

  Socket sock;
  gweMysqlPacket packet;
  String host;
  int port;
  String database;
  String user;
  String password;
  byte protocolVersion;
  String serverVersion;
  int serverMajorVersion;
  int serverMinorVersion;
  int serverSubminorVersion;
  long threadId;
  String cryptSeed;
  long columnCount;
  gweMysqlColumn[] columnResult;
  HashMap columnName;
  HashMap columnFullName;
  int[] dataStart;
  String encoding;
  gweNativeUnicodeMapper mapperclass;

  public gweMysql(String enc, gweNativeUnicodeMapper mapper) {
    encoding = enc;
    mapperclass = mapper;
  }

  // Connect to mysql host
  public synchronized void Connect(String h, int p, String d, String u, String pw) throws gweMysqlException {
    host=h;
    port=p;
    database=d;
    user=u;
    password=pw;
    if (user.equals("")) {user="nobody";}

    try {
      // Open TCP/IP socket
      sock = new Socket(host, port);
      DataInputStream sin = new DataInputStream(sock.getInputStream());
      DataOutputStream sout = new DataOutputStream(sock.getOutputStream());

      // Create new packet handler
      packet = new gweMysqlPacket(sin,sout,encoding,mapperclass);

//      packet.TRACING=true;

      // Get the first packet from mysql
      packet.next();
      protocolVersion = packet.readByte();
      serverVersion = packet.readString(); // major.minor.subminor-??

      parseServerVersion(serverVersion);

      long threadId = packet.read4Bytes();
      cryptSeed = packet.readString();

      // Send who we are
      packet.clear_w();
      packet.write2Bytes(0);
      packet.write3Bytes(packet.MAXBUFFER);
      packet.writeString(user);
      packet.writeString(cryptPassword(password,cryptSeed));
      packet.send();

      // Check if all is ok
      packet.next();
      byte statusCode = packet.readByte();
      if (statusCode==0xff) {
        String errorString = packet.readString();
        throw new gweMysqlException(errorString);
      } else if (statusCode==0x00) {
        long affectedRows = packet.readLength();
        long uniqueId = packet.readLength();
      } else throw new gweMysqlException("Unknown status code '"+statusCode+"'");

      // Ok, we're authenticated, so select our database
      packet.clear_w();
      packet.resetPacketSequence();
      packet.writeByte((byte)gweMysql.INIT_DB);
      packet.writeStringNoNull(database);
      packet.send();

      // Check if all is well
      packet.next();
      statusCode = packet.readByte();
      if (statusCode==0xff) {
        String errorString = packet.readString();
        throw new gweMysqlException(errorString);
      } else if (statusCode==0x00) {
        long affectedRows = packet.readLength();
        long uniqueId = packet.readLength();
      } else throw new gweMysqlException("Unknown status code '"+statusCode+"'");


    } catch (Exception e) {throw new gweMysqlException("gweMysql.Connect: "+e);}
  }

  // major.minor.subminor-??
  void parseServerVersion(String verstr) throws gweMysqlException {
    serverMajorVersion    = -1;
    serverMinorVersion    = -1;
    serverSubminorVersion = -1;
    String temp_verstr;
    int pidx = verstr.indexOf('.');
    if( pidx >= 1 )
    {
      try {
        serverMajorVersion = Integer.parseInt(verstr.substring(0, pidx));
      }
      catch(NumberFormatException ex) {
        throw new gweMysqlException("Illegal Server Version : "+verstr);
      }
    }
    else
    {
      throw new gweMysqlException("Illegal Server Version : "+verstr);
    }

    temp_verstr = verstr.substring(pidx+1, verstr.length());
    pidx = temp_verstr.indexOf('.');
    if( pidx >= 1 )
    {
      try {
        serverMinorVersion = Integer.parseInt(temp_verstr.substring(0, pidx));
      }
      catch(NumberFormatException ex) {
        throw new gweMysqlException("Illegal Server Version : "+verstr);
      }
    }
    else
    {
      throw new gweMysqlException("Illegal Server Version : "+verstr);
    }

    temp_verstr = temp_verstr.substring(pidx+1, temp_verstr.length());
    int lastidx;
    for(lastidx = 0; lastidx < temp_verstr.length(); lastidx++)
    {
      char c = temp_verstr.charAt(lastidx);
      if( c < '0' || c > '9' )
        break;
    }
    try {
      serverSubminorVersion = Integer.parseInt(temp_verstr.substring(0, lastidx));
    }
    catch(NumberFormatException ex) {
      throw new gweMysqlException("Illegal Server Version : "+verstr);
    }
  }

  // return    x > 0 or 0 or x < 0
  int judgeServerVersion(int maj, int min, int sub) {
    if( serverMajorVersion > maj )
      return 1;
    else if( serverMajorVersion < maj )
      return -1;

    if( serverMinorVersion > min )
      return 1;
    else if( serverMinorVersion < min )
      return -1;

    if( serverSubminorVersion > sub )
      return 1;
    else if( serverSubminorVersion < sub )
      return -1;

    return 0;
  }

  String getServerVersion() {
    return serverVersion;
  }
  int getServerMajorVersion() {
    return serverMajorVersion;
  }
  int getServerMinorVersion() {
    return serverMinorVersion;
  }
  int getServerSubminorVersion() {
    return serverSubminorVersion;
  }

  // Return hash for given string
  long hashPassword(String p) {
    long  nr=1345345333;
    long  nr2=7;
    long tmp;
    for (int i=0; i<p.length(); i++) {
    if ((p.charAt(i)==' ') || (p.charAt(i)=='\t') ) continue;
      tmp=(long)p.charAt(i);
      nr^= (((nr & 63)+nr2)*tmp)+ (nr << 8);
      nr2+=tmp;
    }
    return nr & (((long) 1L << 31) -1L);
  }

  // Return encrypted response, given clear password and seed message
  String cryptPassword(String p, String m) {
    if( p == null || p.length() == 0 )
      return p;

    long hp, hm, s1, s2;
    long max_value= 0x01FFFFFF;
    double d;
    byte b;

    hp=this.hashPassword(p);
    hm=this.hashPassword(m);
    long nr=hp^hm;
    nr%=max_value;
    s1=nr;
    s2=nr/2;

    char[] cr = new char[m.length()];
    for (int i=0; i<m.length(); i++) {
      s1=(s1*3+s2)%max_value;
      s2=(s1+s2+33)%max_value;
      d = (double)s1/max_value;
      b = (byte)java.lang.Math.floor((d*31)+64);
      cr[i]=(char)b;
    }
    return new String(cr);
  }

  public gweMysql sqlQuery(String q) throws gweMysqlException {

    // Send query command and sql query string
    packet.clear_w();
    packet.resetPacketSequence();
    packet.writeByte((byte)gweMysql.QUERY);
    packet.writeStringNoNull(q);
    packet.send();

    // Check if we're ok
    packet.next();
    columnCount = packet.readLength();
    if (columnCount==255 || columnCount==-1) { // for NULL FIELD!!
      String errorString = packet.readString();
      throw new gweMysqlException(errorString);
    } else if (columnCount==0) {
      //throw new gweMysqlException("No columns returned.");
      return null;
    } else {
      columnResult = new gweMysqlColumn[(int)columnCount];
      columnName=new HashMap((int)columnCount);
      columnFullName=new HashMap((int)columnCount);
      dataStart = new int[(int)columnCount];

      // Read in the column information
      for (int i=0;i<columnCount; i++) {
        packet.next();
        String tableName=packet.readLenString();
        String colName=packet.readLenString();
        int colLength = packet.readnBytes();
        int colType=packet.readnBytes();
        packet.readByte(); // We know it's currently 2
        short colFlag = (short) packet.ub(packet.readByte());
        int colDecimals = packet.ub(packet.readByte());

        columnResult[i] = new gweMysqlColumn(tableName, colName, colLength, colType, colFlag, colDecimals);
        columnName.put(colName, new Integer(i));
        String fullName = tableName.concat(".").concat(colName);
        columnFullName.put(fullName, new Integer(i));

      }
      packet.next(); //Redundant - we know how many, this is end of section marker packet
    }
    return this;
  }


  public int sqlUpdate(String q) throws gweMysqlException {

    // Send query command and sql update
    packet.clear_w();
    packet.resetPacketSequence();
    packet.writeByte((byte)gweMysql.QUERY);
    packet.writeStringNoNull(q);
    packet.send();

    // check if we're ok
    packet.next();
    int updateStatus = packet.ub(packet.readByte());
    if (updateStatus==255) {
      String errorString = packet.readString();
      throw new gweMysqlException(errorString);
    }
    long affectedRows = packet.readLength();
    long updateId = packet.readLength();

    //return update id
//    return (int)updateId;
    return (int)affectedRows;
  }


  public void sqlQuit() throws gweMysqlException {

    // Send quit command to mysql
    packet.clear_w();
    packet.resetPacketSequence();
    packet.writeByte((byte)gweMysql.QUIT);
    packet.send();

  }



  public boolean nextResult() throws gweMysqlException {
    int i, p;

    // Get the next incoming packet
    packet.next();
    // It it's not the last one, note start positions of data
    if(!packet.isLastDataPacket_r()) {
      for (i=0; i<columnCount; i++) {
        p=packet.currentPos_r();
        dataStart[i]=p;
        switch(packet.ub(packet.getCurDataPos_r()))
        {
          case 252: p += 2; break;
          case 253: p += 3; break;
          case 254: p += 4; break;
          default: break;
        }
        long next_len = packet.readLength();
        if( next_len == -1 )                 // for NULL FIELD!!
          next_len = 0;                      // for NULL FIELD!!
        packet.setPos_r(1+p+(int)next_len);
      }
      return true;
    }
    return false;
  }

  public int getColumnCount() {
    return (int) columnCount;
  }

  public gweMysqlColumn getColumn(int c) throws gweMysqlException {
    c--;  // Columnd go from 1..columnCount to 0..columnCount-1
    if ((c<columnCount)&&(c>=0)) {
      return columnResult[c];
    } else {
      throw new gweMysqlException("Invalid column number : "+c);
    }
  }

  public boolean isNull(int c) throws gweMysqlException {
    //System.out.println("isNull:" + c);
    c--;
    if ((c<columnCount)&&(c>=0)) {
      packet.setPos_r(dataStart[c]);
      long l = packet.readLength();
      return (l==-1); // for NULL FIELD!!
    } else throw new gweMysqlException("Invalid column number : "+c);
  }

  public int getColumnNum(String cn) throws gweMysqlException {
    Integer nI = (Integer) columnName.get(cn);
    if (nI==null) nI = (Integer) columnFullName.get(cn);
    if (nI!=null) return nI.intValue()+1;
    throw new gweMysqlException("Invalid column name : "+cn);
  }


  public String getString(int c) throws gweMysqlException {
    c--;
    if ((c<columnCount)&&(c>=0)) {
      packet.setPos_r(dataStart[c]);
      return packet.readLenString();
    } else throw new gweMysqlException("Invalid column number : "+c);
  }

  public String getString(String cn) throws gweMysqlException {
    Integer nI = (Integer) columnName.get(cn);
    if (nI==null) nI = (Integer) columnFullName.get(cn);
    if (nI!=null) {
      int c = nI.intValue()+1;
      if (c<columnCount) {
        packet.setPos_r(dataStart[c]);
        return packet.readLenString();
      }
    }
    throw new gweMysqlException("Invalid column string : "+cn);
  }

  public byte[] getBytes(int c) throws gweMysqlException {
    c--;
    if ((c<columnCount)&&(c>=0)) {
      packet.setPos_r(dataStart[c]);
      long l = packet.readLength();
      if( l == -1 )                   // for NULL FIELD!!
        return null;                  // for NULL FIELD!!
      //byte[] r = new byte[(int)l];
      //for (int i=0; i<(int)l; i++) {
      //  r[i]=packet.readByte();
      //}
      byte[] r = packet.readByteBlock((int)l);
      return r;
    }
    throw new gweMysqlException("Invalid column number : "+c);
  }


  public void reConnect() throws gweMysqlException {
    try {
      sqlQuit();
      sock.close();
      this.Connect(host, port, database, user, password);
    } catch (Exception e) {throw new gweMysqlException("gweMysql.reConnect: "+e);}
  }


  public void Close() throws gweMysqlException {
    try {
      sqlQuit();
      sock.close();
    } catch (Exception e) {throw new gweMysqlException("gweMysql.Close: "+e);}
  }

  gweMysqlPacket getPacket(){
    return (packet);
  }

  String getEncoding() {
    return encoding;
  }

  gweNativeUnicodeMapper getNativeUnicodeMapper() {
    return mapperclass;
  }
}

