//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 16.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

package gwe.sql;

import java.sql.*;
import java.io.PrintWriter;
import javax.sql.PooledConnection;
import javax.sql.ConnectionPoolDataSource;
import javax.naming.Reference;
import javax.naming.Referenceable;
import java.io.Serializable;
import javax.naming.NamingException;
import javax.naming.StringRefAddr;
import java.util.Properties;
import java.util.LinkedList;
import java.util.ArrayList;

public class gweMysqlConnectionPoolDataSource
                                    implements ConnectionPoolDataSource,
                                               Referenceable, Serializable {

  private static gwe.sql.gweMysqlDriver driver = null;
  private static final String factoryName = "gwe.sql.gweMysqlConnectionPoolDataSourceFactory";
  private int seconds = 0;
  transient private PrintWriter logger = null;
  private String networkProtocol = "jdbc:mysql:";
  private String serverName = "";
  private Integer portNumber = new Integer(3306);
  private String databaseName = "";
  private String dataSourceName = null;
  private String description = "";
  private String user = null;
  private String password = null;
  private String characterEncoding = "Shift_JIS";
  private String nativeUnicodeMapper = null; // nativeUnicodeMapper support!!
  private Boolean add5CAfter5C = new Boolean(false);
  private Integer maxPoolSize = new Integer(0);
  private Boolean debugSQL = new Boolean(false);

  private LinkedList pool = new LinkedList();
  private ArrayList  busy = new ArrayList();

  static {
    try {
      driver = (gwe.sql.gweMysqlDriver)Class.forName(
                                      "gwe.sql.gweMysqlDriver").newInstance();
    }
    catch(Exception e) {
      throw new RuntimeException("Could not Load Driver gwe.sql.gweMysqlDriver class");
    }
  }

  public PooledConnection getPooledConnection() throws java.sql.SQLException {
    return getPooledConnection(getUser(), getPassword());
  }

  public PooledConnection getPooledConnection(String username, String password)
                                                 throws java.sql.SQLException {
    Properties info = new Properties();
    info.setProperty("user", username);
    info.setProperty("password", password);

    if( characterEncoding != null )
      info.setProperty("characterEncoding", characterEncoding);
    if( nativeUnicodeMapper != null )
      info.setProperty("nativeUnicodeMapper", nativeUnicodeMapper);

    info.setProperty("add5CAfter5C", add5CAfter5C.toString());
    info.setProperty("debugSQL", debugSQL.toString());

    String url = networkProtocol + "//" +  serverName + ":" + portNumber + "/"
                 + databaseName;

    return getPooledConnectionSub(url, info);
  }

  public Reference getReference() throws NamingException {
    Reference ref_obj = new Reference(getClass().getName(), factoryName, null);
    ref_obj.add(new StringRefAddr("networkProtocol", getNetworkProtocol()));
    ref_obj.add(new StringRefAddr("serverName",      getServerName()));
    ref_obj.add(new StringRefAddr("portNumber",      getPortNumber().toString()));
    ref_obj.add(new StringRefAddr("databaseName",    getDatabaseName()));
    ref_obj.add(new StringRefAddr("user",            getUser()));
    ref_obj.add(new StringRefAddr("password",        getPassword()));
    ref_obj.add(new StringRefAddr("characterEncoding",
                                                     getCharacterEncoding()));
    ref_obj.add(new StringRefAddr("nativeUnicodeMapper",
                                                     getNativeUnicodeMapper()));
    ref_obj.add(new StringRefAddr("add5CAfter5C",    isAdd5CAfter5C().toString()));
    ref_obj.add(new StringRefAddr("maxPoolSize",     getMaxPoolSize().toString()));
    ref_obj.add(new StringRefAddr("debugSQL",        isDebugSQL().toString()));
    return ref_obj;
  }

  public PooledConnection getPooledConnectionSub(String url, Properties info)
                                                          throws SQLException {
    synchronized(pool) {
      while(true)
      {
        if( pool.size() <= 0 )
        {
          if( maxPoolSize.intValue() <= 0 || busy.size() < maxPoolSize.intValue() )
            break;
          
          while( pool.size() <= 0 )
          {
            try {
              pool.wait();
            }
            catch(InterruptedException itre) {
              throw new SQLException(itre.getMessage());
            }
          }
        }

        gweMysqlPooledConnection pooled_conn =
                                  (gweMysqlPooledConnection)pool.getFirst();
        pool.removeFirst();
        Connection org_conn = pooled_conn.getOrgConnection();
        try {
          org_conn.setAutoCommit(false);
          org_conn.setAutoCommit(true);
          busy.add(pooled_conn);
          return pooled_conn;
        }
        catch(SQLException ex) {
          System.err.println(ex.getMessage());
        }
        try {
          org_conn.close();
        }
        catch(SQLException ex) {
          ;
        }
      }

      gweMysqlPooledConnection ret_conn =
               new gweMysqlPooledConnection(this, driver.connect(url, info));
      busy.add(ret_conn);
      return ret_conn;
    }
  }

  public int getPooledCount() {
    synchronized(pool) {
      return pool.size();
    }
  }

  public void sweepPooledConnections() {
    synchronized(pool) {
      for(int i = 0; i < pool.size(); i++)
      {
        gweMysqlPooledConnection pooled_conn =
                                 (gweMysqlPooledConnection)pool.get(i);
        Connection org_conn = pooled_conn.getOrgConnection();
        try {
          org_conn.close();
        }
        catch(SQLException ex) {
          ;
        }
      }
      pool.clear();
    }
  }

  void storeConnection(gweMysqlPooledConnection conn) {
    synchronized(pool) {
      int i = busy.indexOf(conn);
      if( i >= 0 )
      {
        busy.remove(i);
        pool.addLast(conn);
      }
      pool.notifyAll();
    }
  }

  /**
   * Prints a message to the current JDBC Log writer.
   */
  public synchronized void println(String mes) {
    if( logger != null )
    {
      logger.println(mes);
      logger.flush();
    }
  }

  /**
   *  Set the log writer.
   *
   *  @see javax.sql.DataSource
   */
  public void setLogWriter(PrintWriter logger) throws java.sql.SQLException {
    this.logger = logger;
  }
  public PrintWriter getLogWriter() throws java.sql.SQLException {
    return logger;
  }

  /**
   *  Set the login timeout (in seconds).  If this value is set to
   *  something other than zero, any method that gets a connection
   *  is guaranteed to return (or throw a <tt>SQLException</tt>) within the
   *  specified amount of time.  If this value is set to zero,
   *  any method that gets a connection will block indefinately
   *  until a connection is available.  The default value is zero.
   *
   *  @see javax.sql.DataSource
   */
  public void setLoginTimeout(int seconds) throws java.sql.SQLException {
    this.seconds = seconds;
  }
  public int getLoginTimeout() throws java.sql.SQLException {
    return seconds;
  }

  // An optional database protocol to use when communicating with InterServer.
  // This can only be jdbc:mysql: for MySQL.
  public void setNetworkProtocol(String networkProtocol) {
    this.networkProtocol = networkProtocol;
  }
  public String getNetworkProtocol() {
    return networkProtocol;
  }

  // The server name where MySQL is running.
  public void setServerName(String serverName) {
    this.serverName = serverName;
  }
  public String getServerName() {
    return serverName;
  }

  // The TCP/IP port number where MySQL listens
  // for connection requests on serverName.
  public void setPortNumber(Integer portNumber) {
    this.portNumber = portNumber;
  }
  public Integer getPortNumber() {
    return portNumber;
  }

  // Absolute path to a database on a server.
  public void setDatabaseName(String databaseName) {
    this.databaseName = databaseName;
  }
  public String getDatabaseName() {
    return databaseName;
  }

  // An optional data source name; used to name an underlying XADataSource,
  // or ConnectionPoolDataSource when pooling of connections is done.
  public void setDataSourceName(String dataSourceName) {
    this.dataSourceName = dataSourceName;
  }
  public String getDataSourceName() {
    return dataSourceName;
  }

  // An optional text description of the data source.
  public void setDescription(String description) {
    this.description = description;
  }
  public String getDescription() {
    return description;
  }

  // The user name for the connection.
  public void setUser(String user) {
    this.user = user;
  }
  public String getUser() {
    return user;
  }

  // The user's password for the connection.
  public void setPassword(String password) {
    this.password = password;
  }
  public String getPassword() {
    return password;
  }

  // The character encoding for the connection.
  public void setCharacterEncoding(String characterEncoding) {
    this.characterEncoding = characterEncoding;
  }
  public String getCharacterEncoding() {
    return characterEncoding;
  }

  // The nativeUnicodeMapper for the connection.
  public void setNativeUnicodeMapper(String nativeUnicodeMapper) {
    this.nativeUnicodeMapper = nativeUnicodeMapper;
  }
  public String getNativeUnicodeMapper() {
    return nativeUnicodeMapper;
  }

  // Add 0x5C(\) after 0x5C(\) when you execute PreparedStatement.setString.
  public void setAdd5CAfter5C(Boolean add5CAfter5C) {
    this.add5CAfter5C = add5CAfter5C;
  }
  public Boolean isAdd5CAfter5C() {
    return add5CAfter5C;
  }

  // Max Size of Pooled Connection.
  public void setMaxPoolSize(Integer maxPoolSize) {
    this.maxPoolSize = maxPoolSize;
  }
  public Integer getMaxPoolSize() {
    return maxPoolSize;
  }

  // Trace SQL Statement with LogWriter.
  public void setDebugSQL(Boolean debugSQL) {
    this.debugSQL = debugSQL;
  }
  public Boolean isDebugSQL() {
    return debugSQL;
  }
}
