//////////////////////////////////////////////////////////////////
// mysql JDBC Driver
//
// Version : 0.9.2
//
// JDBC : 1.22
//
// LastModified: 16.03.1997
//
// Copyright (c) 1997 GWE Technologies Ltd. All rights reserved
// See http://gwe.co.uk/mysql/jdbc/copyright.html for full details

package gwe.sql;

import java.sql.Connection;
import java.sql.SQLException;
import java.io.PrintWriter;
import javax.sql.DataSource;
import javax.naming.Reference;
import javax.naming.Referenceable;
import javax.naming.NamingException;
import java.io.Serializable;
import javax.naming.StringRefAddr;
import java.util.Properties;

public class gweMysqlDataSource implements DataSource, Referenceable,
                                                       Serializable {

  private static gwe.sql.gweMysqlDriver driver = null;
  private static final String factoryName = "gwe.sql.gweMysqlDataSourceFactory";
  private int seconds = 0;
  transient private PrintWriter logger = null;
  private String networkProtocol = "jdbc:mysql:";
  private String serverName = "";
  private Integer portNumber = new Integer(3306);
  private String databaseName = "";
  private String dataSourceName = null;
  private String description = "";
  private String user = null;
  private String password = null;
  private String characterEncoding = "Shift_JIS";
  private String nativeUnicodeMapper = null; // nativeUnicodeMapper support!!
  private Boolean add5CAfter5C = new Boolean(false);
  private Boolean debugSQL = new Boolean(false);

  static {
    try {
      driver = (gwe.sql.gweMysqlDriver)Class.forName(
                                      "gwe.sql.gweMysqlDriver").newInstance();
    }
    catch(Exception e) {
      throw new RuntimeException("Could not Load Driver gwe.sql.gweMysqlDriver class");
    }
  }

  public Connection getConnection() throws java.sql.SQLException {
    return getConnection(getUser(), getPassword());
  }

  public Connection getConnection(String username, String password)
                                                throws java.sql.SQLException {
    Properties info = new Properties();
    info.setProperty("user", username);
    info.setProperty("password", password);

    if( characterEncoding != null )
      info.setProperty("characterEncoding", characterEncoding);
    if( nativeUnicodeMapper != null )
      info.setProperty("nativeUnicodeMapper", nativeUnicodeMapper);

    info.setProperty("add5CAfter5C", add5CAfter5C.toString());
    info.setProperty("debugSQL", debugSQL.toString());

    String url = networkProtocol + "//" +  serverName + ":" + portNumber + "/"
                 + databaseName;

    return driver.connect(url, info);
  }

  public Reference getReference() throws NamingException {
    Reference ref_obj = new Reference(getClass().getName(), factoryName, null);
    ref_obj.add(new StringRefAddr("networkProtocol", getNetworkProtocol()));
    ref_obj.add(new StringRefAddr("serverName",      getServerName()));
    ref_obj.add(new StringRefAddr("portNumber",      getPortNumber().toString()));
    ref_obj.add(new StringRefAddr("databaseName",    getDatabaseName()));
    ref_obj.add(new StringRefAddr("user",            getUser()));
    ref_obj.add(new StringRefAddr("password",        getPassword()));
    ref_obj.add(new StringRefAddr("characterEncoding",
                                                     getCharacterEncoding()));
    ref_obj.add(new StringRefAddr("nativeUnicodeMapper",
                                                     getNativeUnicodeMapper()));
    ref_obj.add(new StringRefAddr("add5CAfter5C",    isAdd5CAfter5C().toString()));
    ref_obj.add(new StringRefAddr("debugSQL",        isDebugSQL().toString()));
    return ref_obj;
  }

  /**
   * Prints a message to the current JDBC Log writer.
   */
  public synchronized void println(String mes) {
    if( logger != null )
    {
      logger.println(mes);
      logger.flush();
    }
  }

  public void setLogWriter(PrintWriter logger) throws java.sql.SQLException {
    this.logger = logger;
  }
  public PrintWriter getLogWriter() throws java.sql.SQLException {
    return logger;
  }

  public void setLoginTimeout(int seconds) throws java.sql.SQLException {
    this.seconds = seconds;
  }
  public int getLoginTimeout() throws java.sql.SQLException {
    return seconds;
  }

  // An optional database protocol to use when communicating with InterServer.
  // This can only be jdbc:mysql: for MySQL.
  public void setNetworkProtocol(String networkProtocol) {
    this.networkProtocol = networkProtocol;
  }
  public String getNetworkProtocol() {
    return networkProtocol;
  }

  // The server name where MySQL is running.
  public void setServerName(String serverName) {
    this.serverName = serverName;
  }
  public String getServerName() {
    return serverName;
  }

  // The TCP/IP port number where MySQL listens
  // for connection requests on serverName.
  public void setPortNumber(Integer portNumber) {
    this.portNumber = portNumber;
  }
  public Integer getPortNumber() {
    return portNumber;
  }

  // Absolute path to a database on a server.
  public void setDatabaseName(String databaseName) {
    this.databaseName = databaseName;
  }
  public String getDatabaseName() {
    return databaseName;
  }

  // An optional data source name; used to name an underlying XADataSource,
  // or ConnectionPoolDataSource when pooling of connections is done.
  public void setDataSourceName(String dataSourceName) {
    this.dataSourceName = dataSourceName;
  }
  public String getDataSourceName() {
    return dataSourceName;
  }

  // An optional text description of the data source.
  public void setDescription(String description) {
    this.description = description;
  }
  public String getDescription() {
    return description;
  }

  // The user name for the connection.
  public void setUser(String user) {
    this.user = user;
  }
  public String getUser() {
    return user;
  }

  // The user's password for the connection.
  public void setPassword(String password) {
    this.password = password;
  }
  public String getPassword() {
    return password;
  }

  // The character encoding for the connection.
  public void setCharacterEncoding(String characterEncoding) {
    this.characterEncoding = characterEncoding;
  }
  public String getCharacterEncoding() {
    return characterEncoding;
  }

  // The nativeUnicodeMapper for the connection.
  public void setNativeUnicodeMapper(String nativeUnicodeMapper) {
    this.nativeUnicodeMapper = nativeUnicodeMapper;
  }
  public String getNativeUnicodeMapper() {
    return nativeUnicodeMapper;
  }

  // Add 0x5C(\) after 0x5C(\) when you execute PreparedStatement.setString.
  public void setAdd5CAfter5C(Boolean add5CAfter5C) {
    this.add5CAfter5C = add5CAfter5C;
  }
  public Boolean isAdd5CAfter5C() {
    return add5CAfter5C;
  }

  // Trace SQL Statement with LogWriter.
  public void setDebugSQL(Boolean debugSQL) {
    this.debugSQL = debugSQL;
  }
  public Boolean isDebugSQL() {
    return debugSQL;
  }
}
